import pandas as pd
import os
from argparse import ArgumentParser

def mergeDataFrames(path="Logs", outFilename="combined_summary.csv"):
    """Merge all of the dataframe csv files in a path, storing the parameter
    values that are encoded in their names as extra columns. Save the merged
    file in the same directory and delete all of the composite files.
    =========
    Arguments
    ---------
    path: the path to the directory in which the csv dataframe files are stored.
    outFilename: the name to give the merged file.
    """
    
    #Get the list of files in the directory
    filenames = os.listdir(path)
    
    #Only proceed if it looks like the merge has not already been applied
    if outFilename not in filenames:
    
        #Individual dataframes will be stored in a list and then concatenated
        dataFrames = list()
        
        #Iterate through csv files in the path
        for filename in filenames:
            if filename.endswith(".csv"):
                
                #Read the csv file as a dataframe
                try:
                    with open("%s/%s" % (path, filename), mode="r", encoding="utf-8") as inFile:
                        df = pd.read_csv(inFile)
                except:
                    try:
                        with open("%s/%s" % (path, filename), mode="rb") as inFile:
                            df = pd.read_csv(inFile)
                    except:
                        raise Exception("Error reading file %s" % filename)
                    
                #Split the filename to get parameter values
                splitFilename = filename.split("_")
                
                #Add parameter values to dataframe
                for paramPair in splitFilename[:-1]:
                    paramName, paramValue = paramPair.split("=")
                    df[paramName] = paramValue
                    
                #Add the dataframe to the list
                dataFrames.append(df)
            
        #Concatenate all dataframes in the list
        combinedDataFrame = pd.concat(dataFrames, ignore_index=True).fillna("")
        
        try:
            #Write the merged dataframe to csv
            combinedDataFrame.to_csv("%s/%s" % (path, outFilename), mode="w", encoding='utf-8', index=False)
            
            #Delete all original csv files
            for filename in filenames:
                if filename.endswith(".csv"):
                    os.remove("%s/%s" % (path, filename))
            
        except:
            raise Exception("Could not write to csv")
    

if __name__ == "__main__":
    
    #Arguments to read from the command line
    parser=ArgumentParser()
    parser.add_argument('--dir', help="Directory to process", type=str, default="Logs")
    parser.add_argument('--outcsv', help="Name of output csv file", type=str, default="combined_summary.csv")
    args= parser.parse_args()
    
    #Merge dataframes when called from the command line
    mergeDataFrames(path = args.dir, outFilename = args.outcsv)